@@ -10,6 +10,7 @@ from __future__ import division
 import collections
 import contextlib
 import errno
+import glob
 import io
 import os
 import re
@@ -24,6 +25,7 @@ import warnings
 import psutil
 from psutil import LINUX
 from psutil._compat import basestring
+from psutil._compat import FileNotFoundError
 from psutil._compat import PY3
 from psutil._compat import u
 from psutil.tests import call_until
@@ -54,7 +56,7 @@ SIOCGIFCONF = 0x8912
 SIOCGIFHWADDR = 0x8927
 if LINUX:
     SECTOR_SIZE = 512
-
+EMPTY_TEMPERATURES = not glob.glob('/sys/class/hwmon/hwmon*')
 
 # =====================================================================
 # --- utils
@@ -696,11 +698,6 @@ class TestSystemCPUCountPhysical(unittest.TestCase):
 @unittest.skipIf(not LINUX, "LINUX only")
 class TestSystemCPUFrequency(unittest.TestCase):
 
-    @unittest.skipIf(not HAS_CPU_FREQ, "not supported")
-    def test_emulate_no_files(self):
-        with mock.patch("os.path.exists", return_value=False):
-            self.assertIsNone(psutil.cpu_freq())
-
     @unittest.skipIf(TRAVIS, "fails on Travis")
     @unittest.skipIf(not HAS_CPU_FREQ, "not supported")
     def test_emulate_use_second_file(self):
@@ -709,15 +706,12 @@ class TestSystemCPUFrequency(unittest.TestCase):
             if path.startswith("/sys/devices/system/cpu/cpufreq/policy"):
                 return False
             else:
-                flags.append(None)
                 return orig_exists(path)
 
-        flags = []
         orig_exists = os.path.exists
         with mock.patch("os.path.exists", side_effect=path_exists_mock,
                         create=True):
             assert psutil.cpu_freq()
-            self.assertEqual(len(flags), psutil.cpu_count(logical=True))
 
     @unittest.skipIf(not HAS_CPU_FREQ, "not supported")
     def test_emulate_use_cpuinfo(self):
@@ -760,6 +754,8 @@ class TestSystemCPUFrequency(unittest.TestCase):
             elif (name.endswith('/scaling_max_freq') and
                     name.startswith("/sys/devices/system/cpu/cpufreq/policy")):
                 return io.BytesIO(b"700000")
+            elif name == '/proc/cpuinfo':
+                return io.BytesIO(b"cpu MHz		: 500")
             else:
                 return orig_open(name, *args, **kwargs)
 
@@ -770,8 +766,12 @@ class TestSystemCPUFrequency(unittest.TestCase):
                     'os.path.exists', return_value=True):
                 freq = psutil.cpu_freq()
                 self.assertEqual(freq.current, 500.0)
-                self.assertEqual(freq.min, 600.0)
-                self.assertEqual(freq.max, 700.0)
+                # when /proc/cpuinfo is used min and max frequencies are not
+                # available and are set to 0.
+                if freq.min != 0.0:
+                    self.assertEqual(freq.min, 600.0)
+                if freq.max != 0.0:
+                    self.assertEqual(freq.max, 700.0)
 
     @unittest.skipIf(not HAS_CPU_FREQ, "not supported")
     def test_emulate_multi_cpu(self):
@@ -795,6 +795,9 @@ class TestSystemCPUFrequency(unittest.TestCase):
             elif (n.endswith('/scaling_max_freq') and
                     n.startswith("/sys/devices/system/cpu/cpufreq/policy1")):
                 return io.BytesIO(b"600000")
+            elif name == '/proc/cpuinfo':
+                return io.BytesIO(b"cpu MHz		: 100\n"
+                                  b"cpu MHz		: 400")
             else:
                 return orig_open(name, *args, **kwargs)
 
@@ -806,11 +809,15 @@ class TestSystemCPUFrequency(unittest.TestCase):
                                 return_value=2):
                     freq = psutil.cpu_freq(percpu=True)
                     self.assertEqual(freq[0].current, 100.0)
-                    self.assertEqual(freq[0].min, 200.0)
-                    self.assertEqual(freq[0].max, 300.0)
+                    if freq[0].min != 0.0:
+                        self.assertEqual(freq[0].min, 200.0)
+                    if freq[0].max != 0.0:
+                        self.assertEqual(freq[0].max, 300.0)
                     self.assertEqual(freq[1].current, 400.0)
-                    self.assertEqual(freq[1].min, 500.0)
-                    self.assertEqual(freq[1].max, 600.0)
+                    if freq[1].min != 0.0:
+                        self.assertEqual(freq[1].min, 500.0)
+                    if freq[1].max != 0.0:
+                        self.assertEqual(freq[1].max, 600.0)
 
     @unittest.skipIf(TRAVIS, "fails on Travis")
     @unittest.skipIf(not HAS_CPU_FREQ, "not supported")
@@ -821,6 +828,8 @@ class TestSystemCPUFrequency(unittest.TestCase):
                 raise IOError(errno.ENOENT, "")
             elif name.endswith('/cpuinfo_cur_freq'):
                 return io.BytesIO(b"200000")
+            elif name == '/proc/cpuinfo':
+                return io.BytesIO(b"cpu MHz		: 200")
             else:
                 return orig_open(name, *args, **kwargs)
 
@@ -833,23 +842,6 @@ class TestSystemCPUFrequency(unittest.TestCase):
                     freq = psutil.cpu_freq()
                     self.assertEqual(freq.current, 200)
 
-        # Also test that NotImplementedError is raised in case no
-        # current freq file is present.
-
-        def open_mock(name, *args, **kwargs):
-            if name.endswith('/scaling_cur_freq'):
-                raise IOError(errno.ENOENT, "")
-            elif name.endswith('/cpuinfo_cur_freq'):
-                raise IOError(errno.ENOENT, "")
-            else:
-                return orig_open(name, *args, **kwargs)
-
-        orig_open = open
-        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
-        with mock.patch(patch_point, side_effect=open_mock):
-            with mock.patch('os.path.exists', return_value=True):
-                self.assertRaises(NotImplementedError, psutil.cpu_freq)
-
 
 @unittest.skipIf(not LINUX, "LINUX only")
 class TestSystemCPUStats(unittest.TestCase):
@@ -1068,10 +1060,9 @@ class TestSystemDiskPartitions(unittest.TestCase):
         try:
             with mock.patch('os.path.realpath',
                             return_value='/non/existent') as m:
-                with self.assertRaises(OSError) as cm:
+                with self.assertRaises(FileNotFoundError):
                     psutil.disk_partitions()
                 assert m.called
-                self.assertEqual(cm.exception.errno, errno.ENOENT)
         finally:
             psutil.PROCFS_PATH = "/proc"
 
@@ -1557,6 +1548,7 @@ class TestSensorsBattery(unittest.TestCase):
 class TestSensorsTemperatures(unittest.TestCase):
 
     @unittest.skipIf(TRAVIS, "unreliable on TRAVIS")
+    @unittest.skipIf(LINUX and EMPTY_TEMPERATURES, "no temperatures")
     def test_emulate_eio_error(self):
         def open_mock(name, *args, **kwargs):
             if name.endswith("_input"):
@@ -1908,9 +1900,8 @@ class TestProcess(unittest.TestCase):
                 '/proc/%s/smaps' % os.getpid(),
                 IOError(errno.ENOENT, "")) as m:
             p = psutil.Process()
-            with self.assertRaises(IOError) as err:
+            with self.assertRaises(FileNotFoundError):
                 p.memory_maps()
-            self.assertEqual(err.exception.errno, errno.ENOENT)
             assert m.called
 
     @unittest.skipIf(not HAS_RLIMIT, "not supported")
@@ -1982,6 +1973,9 @@ class TestProcess(unittest.TestCase):
             "0",      # cnswap
             "0",      # exit_signal
             "6",      # processor
+            "0",      # rt priority
+            "0",      # policy
+            "7",      # delayacct_blkio_ticks
         ]
         content = " ".join(args).encode()
         with mock_open_content('/proc/%s/stat' % os.getpid(), content):
@@ -1996,6 +1990,7 @@ class TestProcess(unittest.TestCase):
             self.assertEqual(cpu.system, 3 / CLOCK_TICKS)
             self.assertEqual(cpu.children_user, 4 / CLOCK_TICKS)
             self.assertEqual(cpu.children_system, 5 / CLOCK_TICKS)
+            self.assertEqual(cpu.iowait, 7 / CLOCK_TICKS)
             self.assertEqual(p.cpu_num(), 6)
 
     def test_status_file_parsing(self):
